/**
 * Copyright (C) 2010 by alexei (Alexei Krasnopolski)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package com.ak.erlang;

import java.io.IOException;

import org.apache.log4j.Logger;

import com.ericsson.otp.erlang.OtpErlangAtom;
import com.ericsson.otp.erlang.OtpErlangException;
import com.ericsson.otp.erlang.OtpErlangList;
import com.ericsson.otp.erlang.OtpErlangLong;
import com.ericsson.otp.erlang.OtpErlangObject;
import com.ericsson.otp.erlang.OtpErlangTuple;
import com.ericsson.otp.erlang.OtpMbox;
import com.ericsson.otp.erlang.OtpNode;

/**
 * The ErlangJavaAdapter class.
 *
 * @version 1.0.0
 * @author <a href="mailto:krasnop@bellsouth.net">Alexei Krasnopolski</a>
 */
public class ErlangJavaAdapter extends OtpNode {
	/** Log4J logger. */
	protected static final Logger LOG = Logger.getLogger(ErlangJavaAdapter.class.getSimpleName());
	/** Local mail box. */
	private OtpMbox mBox;
	/** Request to send to remote Erlang node. */
	private OtpErlangTuple request;
	/** Short name of Erlang remote node. */
	private String erlangNode;
	/** DNS name of remote host. */
	private String erlangHost;
	/** Name of registered Erlang process. */
	private String erlangProcess;

	/**
	 * Constructor.
	 * @param javaHost - DNS name of local host where the JVM resides.
	 * @throws IOException - exception.
	 */
	public ErlangJavaAdapter(final String javaHost) throws IOException {
		super("javaNode@" + javaHost);
		mBox = createMbox();
		request = new OtpErlangTuple(
				new OtpErlangObject [] {
					new OtpErlangAtom("status"),
					mBox.self()
				}
			);
	}

	/**
	 * Get states of philosophers from Erlang server.
	 * @return - array of philosopher's states
	 */
	public String [] getStates() {
		String [] states = new String [0];
		mBox.send(erlangProcess, erlangNode + "@" + erlangHost, request);
		OtpErlangTuple response;
		try {
			response = (OtpErlangTuple) mBox.receive(1000);
			if (response == null) {
				throw new OtpErlangException() {
					@Override
					public String getMessage() {
						return "Timeout...";
					}
				};
			}
			int n = ((OtpErlangLong) (response.elementAt(1))).intValue();
			OtpErlangList list = (OtpErlangList) (response.elementAt(2));
			states = new String [n];

			for (OtpErlangObject obj : list.elements()) {
				OtpErlangTuple tuple = (OtpErlangTuple) obj;
				int idx = ((OtpErlangLong) (tuple.elementAt(0))).intValue() - 1;
				states[idx] = ((OtpErlangAtom) (tuple.elementAt(1))).atomValue();
			}
		} catch (OtpErlangException e) {
			LOG.error("Error while connect to Erlang server:", e);
		}
		return states;
	}

	/**
	 * @param erlangNode the erlangNode to set
	 */
	public void setErlangNode(final String erlangNode) {
		this.erlangNode = erlangNode;
	}

	/**
	 * @param erlangHost the erlangHost to set
	 */
	public void setErlangHost(final String erlangHost) {
		this.erlangHost = erlangHost;
	}

	/**
	 * @param erlangProcess the erlangProcess to set
	 */
	public void setErlangProcess(final String erlangProcess) {
		this.erlangProcess = erlangProcess;
	}
}
